import os
import pytest
import random
import string
from typing import Any, Tuple

import vegaapiclient as vac


@pytest.fixture(scope="module")
def trading():
    """
    Provide a VegaTradingClient, connected to $GRPC_NODE.
    """
    grpc_node = os.getenv("GRPC_NODE")
    assert grpc_node is not None and grpc_node != ""
    return vac.VegaTradingClient(grpc_node)


@pytest.fixture(scope="module")
def tradingdata():
    """
    Provide a VegaTradingDataClient, connected to $GRPC_NODE.
    """
    grpc_node = os.getenv("GRPC_NODE")
    assert grpc_node is not None and grpc_node != ""
    return vac.VegaTradingDataClient(grpc_node)


@pytest.fixture(scope="module")
def faucetclient():
    """
    Provide a FaucetClient, connected to $FAUCETSERVER.
    """
    faucetserver = os.getenv("FAUCETSERVER")
    assert faucetserver is not None and faucetserver != ""
    return vac.FaucetClient(faucetserver)


@pytest.fixture(scope="module")
def walletclient():
    """
    Provide a WalletClient, connected to $WALLETSERVER.
    """
    walletserver = os.getenv("WALLETSERVER")
    assert walletserver is not None and walletserver != ""
    return vac.WalletClient(walletserver)


@pytest.fixture(scope="module")
def walletname() -> str:
    """
    Return a random wallet name.
    """
    choices = string.ascii_letters + string.digits
    return "".join(random.choice(choices) for i in range(40))


@pytest.fixture(scope="module")
def walletpassphrase() -> str:
    """
    Return a random wallet passphrase, passing strength requirements.
    """
    return "".join(
        "".join(random.choice(req) for i in range(5))
        for req in [
            string.ascii_uppercase,
            string.ascii_lowercase,
            string.digits,
            string.punctuation,
        ]
    )


@pytest.fixture(scope="module")
def walletClientWalletKeypair(
    walletclient, walletname, walletpassphrase
) -> Tuple[Any, str, str, str]:
    """
    Provide a WalletClient that has had a wallet and keypair added.

    Returns:
    * a WalletClient
    * a random wallet name
    * a random passphrase
    * a public key
    """

    # Create wallet
    r = walletclient.create(walletname, walletpassphrase)
    assert r.status_code == 200
    j = r.json()
    assert "token" in j, "Bad response from CreateWallet: {}".format(j)

    # Create a keypair
    r = walletclient.generatekey(walletpassphrase, [])
    assert r.status_code == 200
    pubKey = r.json()["key"]["pub"]

    return (walletclient, walletname, walletpassphrase, pubKey)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/assets.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/assets.proto',
  package='vega',
  syntax='proto3',
  serialized_options=b'Z\037code.vegaprotocol.io/vega/proto',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12proto/assets.proto\x12\x04vega\"{\n\x05\x41sset\x12\n\n\x02ID\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0e\n\x06symbol\x18\x03 \x01(\t\x12\x13\n\x0btotalSupply\x18\x04 \x01(\t\x12\x10\n\x08\x64\x65\x63imals\x18\x05 \x01(\x04\x12!\n\x06source\x18\x07 \x01(\x0b\x32\x11.vega.AssetSource\"a\n\x0b\x41ssetSource\x12*\n\x0c\x62uiltinAsset\x18\x01 \x01(\x0b\x32\x12.vega.BuiltinAssetH\x00\x12\x1c\n\x05\x65rc20\x18\x02 \x01(\x0b\x32\x0b.vega.ERC20H\x00\x42\x08\n\x06source\"p\n\x0c\x42uiltinAsset\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06symbol\x18\x02 \x01(\t\x12\x13\n\x0btotalSupply\x18\x03 \x01(\t\x12\x10\n\x08\x64\x65\x63imals\x18\x04 \x01(\x04\x12\x1b\n\x13maxFaucetAmountMint\x18\x05 \x01(\t\" \n\x05\x45RC20\x12\x17\n\x0f\x63ontractAddress\x18\x01 \x01(\t\"/\n\tDevAssets\x12\"\n\x07sources\x18\x01 \x03(\x0b\x32\x11.vega.AssetSourceB!Z\x1f\x63ode.vegaprotocol.io/vega/protob\x06proto3'
)




_ASSET = _descriptor.Descriptor(
  name='Asset',
  full_name='vega.Asset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ID', full_name='vega.Asset.ID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='vega.Asset.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='symbol', full_name='vega.Asset.symbol', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='totalSupply', full_name='vega.Asset.totalSupply', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='decimals', full_name='vega.Asset.decimals', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source', full_name='vega.Asset.source', index=5,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=28,
  serialized_end=151,
)


_ASSETSOURCE = _descriptor.Descriptor(
  name='AssetSource',
  full_name='vega.AssetSource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='builtinAsset', full_name='vega.AssetSource.builtinAsset', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='erc20', full_name='vega.AssetSource.erc20', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='source', full_name='vega.AssetSource.source',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=153,
  serialized_end=250,
)


_BUILTINASSET = _descriptor.Descriptor(
  name='BuiltinAsset',
  full_name='vega.BuiltinAsset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='vega.BuiltinAsset.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='symbol', full_name='vega.BuiltinAsset.symbol', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='totalSupply', full_name='vega.BuiltinAsset.totalSupply', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='decimals', full_name='vega.BuiltinAsset.decimals', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='maxFaucetAmountMint', full_name='vega.BuiltinAsset.maxFaucetAmountMint', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=252,
  serialized_end=364,
)


_ERC20 = _descriptor.Descriptor(
  name='ERC20',
  full_name='vega.ERC20',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='contractAddress', full_name='vega.ERC20.contractAddress', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=366,
  serialized_end=398,
)


_DEVASSETS = _descriptor.Descriptor(
  name='DevAssets',
  full_name='vega.DevAssets',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sources', full_name='vega.DevAssets.sources', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=400,
  serialized_end=447,
)

_ASSET.fields_by_name['source'].message_type = _ASSETSOURCE
_ASSETSOURCE.fields_by_name['builtinAsset'].message_type = _BUILTINASSET
_ASSETSOURCE.fields_by_name['erc20'].message_type = _ERC20
_ASSETSOURCE.oneofs_by_name['source'].fields.append(
  _ASSETSOURCE.fields_by_name['builtinAsset'])
_ASSETSOURCE.fields_by_name['builtinAsset'].containing_oneof = _ASSETSOURCE.oneofs_by_name['source']
_ASSETSOURCE.oneofs_by_name['source'].fields.append(
  _ASSETSOURCE.fields_by_name['erc20'])
_ASSETSOURCE.fields_by_name['erc20'].containing_oneof = _ASSETSOURCE.oneofs_by_name['source']
_DEVASSETS.fields_by_name['sources'].message_type = _ASSETSOURCE
DESCRIPTOR.message_types_by_name['Asset'] = _ASSET
DESCRIPTOR.message_types_by_name['AssetSource'] = _ASSETSOURCE
DESCRIPTOR.message_types_by_name['BuiltinAsset'] = _BUILTINASSET
DESCRIPTOR.message_types_by_name['ERC20'] = _ERC20
DESCRIPTOR.message_types_by_name['DevAssets'] = _DEVASSETS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Asset = _reflection.GeneratedProtocolMessageType('Asset', (_message.Message,), {
  'DESCRIPTOR' : _ASSET,
  '__module__' : 'proto.assets_pb2'
  # @@protoc_insertion_point(class_scope:vega.Asset)
  })
_sym_db.RegisterMessage(Asset)

AssetSource = _reflection.GeneratedProtocolMessageType('AssetSource', (_message.Message,), {
  'DESCRIPTOR' : _ASSETSOURCE,
  '__module__' : 'proto.assets_pb2'
  # @@protoc_insertion_point(class_scope:vega.AssetSource)
  })
_sym_db.RegisterMessage(AssetSource)

BuiltinAsset = _reflection.GeneratedProtocolMessageType('BuiltinAsset', (_message.Message,), {
  'DESCRIPTOR' : _BUILTINASSET,
  '__module__' : 'proto.assets_pb2'
  # @@protoc_insertion_point(class_scope:vega.BuiltinAsset)
  })
_sym_db.RegisterMessage(BuiltinAsset)

ERC20 = _reflection.GeneratedProtocolMessageType('ERC20', (_message.Message,), {
  'DESCRIPTOR' : _ERC20,
  '__module__' : 'proto.assets_pb2'
  # @@protoc_insertion_point(class_scope:vega.ERC20)
  })
_sym_db.RegisterMessage(ERC20)

DevAssets = _reflection.GeneratedProtocolMessageType('DevAssets', (_message.Message,), {
  'DESCRIPTOR' : _DEVASSETS,
  '__module__' : 'proto.assets_pb2'
  # @@protoc_insertion_point(class_scope:vega.DevAssets)
  })
_sym_db.RegisterMessage(DevAssets)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

VERSION = "0.26.0-pre1"

setuptools.setup(
    name="Vega API client",
    version=VERSION,
    author="Vega",
    author_email="hi@vega.xyz",
    description="Vega API client for gRPC",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/vegaprotocol/api-clients",
    packages=setuptools.find_packages(),
    package_data={"vegaapiclient": ["py.typed"],},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=["googleapis-common-protos", "grpcio", "requests",],
    setup_requires=["wheel"],
    zip_safe=False,
)



import base64
import time
from google.protobuf.empty_pb2 import Empty

import vegaapiclient as vac

from .fixtures import (  # noqa: F401
    trading,
    tradingdata,
    faucetclient,
    walletclient,
    walletClientWalletKeypair,
    walletname,
    walletpassphrase,
)


def test_SubmitOrder(
    trading, tradingdata, walletClientWalletKeypair, faucetclient  # noqa: F811
):
    (walletclient, _, _, pubKey) = walletClientWalletKeypair  # noqa: F811

    # Get a market
    markets = tradingdata.Markets(Empty()).markets
    assert len(markets) > 0
    market = markets[0]

    # Mint some tokens in the market's settlement token
    amt = 10
    assetID = market.tradableInstrument.instrument.future.asset
    print(f"{pubKey}: Minting {amt} {assetID}")
    mintresponse = faucetclient.mint(amt, assetID, pubKey)
    assert mintresponse.status_code == 200
    assert mintresponse.json()["success"]

    # Wait until funds are in accounts
    x = 0
    maxwait = 300  # seconds
    while True:
        request = vac.api.trading.PartyAccountsRequest(
            partyID=pubKey,
            marketID=market.id,
            type=vac.vega.AccountType.ACCOUNT_TYPE_GENERAL,
            asset=assetID,
        )
        response = tradingdata.PartyAccounts(request)
        n = len(response.accounts)
        print(f"Account count: {n}")
        if n > 0:
            print(f"Accounts: {response.accounts}")
        if n == 1 and response.accounts[0].balance == amt:
            # Log time taken
            # with open("/tmp/log", "a") as fh:
            #     fh.write(f"{x}\n")
            break
        if x >= maxwait:
            assert (
                False
            ), f"Failed to see {amt} {assetID} in {pubKey}'s general account"
        time.sleep(1)
        x += 1

    # Prepare the SubmitOrder
    now = int(tradingdata.GetVegaTime(Empty()).timestamp)
    request = vac.api.trading.SubmitOrderRequest(
        submission=vac.vega.OrderSubmission(
            expiresAt=now + 120000000000,
            marketID=market.id,
            partyID=pubKey,
            price=10,
            side=vac.vega.Side.SIDE_BUY,
            size=1,
            timeInForce=vac.vega.Order.TimeInForce.TIF_GTT,
            type=vac.vega.Order.Type.TYPE_LIMIT,
        )
    )
    response = trading.PrepareSubmitOrder(request)
    blob = response.blob

    # Sign the tx
    r = walletclient.signtx(base64.b64encode(blob).decode("ascii"), pubKey, False)
    assert r.status_code == 200
    signedTx = r.json()["signedTx"]

    # Submit the signed transaction
    # assert blob == base64.b64decode(signedTx["data"])
    request = vac.api.trading.SubmitTransactionRequest(
        tx=vac.vega.SignedBundle(
            tx=base64.b64decode(signedTx["tx"]),
            sig=vac.vega.Signature(
                sig=base64.b64decode(signedTx["sig"]["sig"]),
                algo=signedTx["sig"]["algo"],
                version=signedTx["sig"]["version"],
            ),
        )
    )
    response = trading.SubmitTransaction(request)
    assert response.success
